#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Oct 27 17:16:54 2019

@author: François Pelletier
"""

import nltk
import re
import math
from collections import defaultdict
from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer
from nltk.corpus import wordnet as wn
from nltk.corpus import sentiwordnet as swn
from sklearn.naive_bayes import MultinomialNB
from sklearn.linear_model import LogisticRegression

# Normalisation

def tokenize(reviews):
    tokenizer = nltk.tokenize.ToktokTokenizer()
    tokenized_reviews = []
    for review in reviews:
        # Plusieurs fin de phrases étaient représentées par deux espaces ou plus.
        review = re.sub(r"\s{2,}", ". ", review)
        review = str.lower(review)
        tokenized_sentences = []
        sentences = nltk.sent_tokenize(review)
        for sentence in sentences:
            sentence_tokens = tokenizer.tokenize(sentence)
            # Suppression de la ponctuation
            for token in sentence_tokens:
                if token in "?:!.,;":
                    sentence_tokens.remove(token)
            tokenized_sentences.append(sentence_tokens)
        tokenized_reviews.append(tokenized_sentences)
    return tokenized_reviews


def norm_stemming(tokenized_reviews):
    porter = nltk.PorterStemmer()
    stemmed_reviews = []
    for review in tokenized_reviews:
        stemmed_review = []
        for sentence in review:
            stemmed_tokens = [porter.stem(token) for token in sentence]
            stemmed_review.append(stemmed_tokens)
        stemmed_reviews.append(stemmed_review)
    return stemmed_reviews


def norm_lemmatize(tokenized_reviews):
    wnl = nltk.WordNetLemmatizer()
    normalized_reviews = []
    for review in tokenized_reviews:
        normalized_review = []
        for sentence in review:
            # Lemmatisation au verbe et non au nom
            normalized_tokens = [wnl.lemmatize(token, pos="v") for token in sentence]
            normalized_review.append(normalized_tokens)
        normalized_reviews.append(normalized_review)
    return normalized_reviews


# Feature selection

def dict_frequency(norm_reviews):
    tokens_frequency = defaultdict(int)
    for review in norm_reviews:
        for sentence in review:
            for token in sentence:
                tokens_frequency[token] += 1
    return tokens_frequency


def select_freq(norm_reviews):
    tokens_frequency = dict_frequency(norm_reviews)
    norm_reviews_freq3 = [
        [
            [token for token in sentence if tokens_frequency[token] > 3]
            for sentence in review
        ]
        for review in norm_reviews
    ]
    return norm_reviews_freq3


def select_rem_stopwords(norm_reviews):
    sws = set(nltk.corpus.stopwords.words('english'))
    norm_reviews_stoprem = [
        [
            [token for token in sentence if token not in sws]
            for sentence in review
        ]
        for review in norm_reviews
    ]
    return norm_reviews_stoprem


def pos_tag_reviews(norm_reviews):
    tagged_reviews = []
    for review in norm_reviews:
        tagged_review = nltk.pos_tag_sents(review, tagset='universal')
        tagged_reviews.append(tagged_review)
    return tagged_reviews


def validate_open_class(token_tuple):
    # i is not tagged right
    if token_tuple[1] in ['NOUN','ADJ','ADV','VERB'] and token_tuple[0] != 'i':
        return True
    else:
        return False


def select_open_class(norm_reviews):
    tagged_reviews = pos_tag_reviews(norm_reviews)
    select_oc_token = [
        [
            [token_tuple[0] for token_tuple in sentence if validate_open_class(token_tuple)]
            for sentence in review
        ]
        for review in tagged_reviews
    ]
    return select_oc_token


# Attribute value

def dummy_function(x):
    return x


def corpus_documents(corpus):
    corpus_documents = []
    for document in corpus:
        doc_sentences = []
        for sentence in document:
            doc_sentences += sentence
        corpus_documents.append(doc_sentences)
    return corpus_documents


def get_vectorizers(corpus_documents):
    cvr = CountVectorizer(analyzer='word', 
                          tokenizer=dummy_function, 
                          preprocessor=dummy_function)
    hvr = CountVectorizer(analyzer='word', 
                            tokenizer=dummy_function, 
                            preprocessor=dummy_function,
                            binary=True)
    tdvr = TfidfVectorizer(analyzer='word', 
                          tokenizer=dummy_function, 
                          preprocessor=dummy_function)
    
    cvr.fit(corpus_documents)
    hvr.fit(corpus_documents)    
    tdvr.fit(corpus_documents)
    return [cvr,hvr,tdvr]

# Other attributes

def attribute_polarity_count(norm_reviews):
    polarity = []
    for review in norm_reviews:
        polarity_pos=0
        polarity_neg=0
        word_count=0
        for sentence in review:
            for word in sentence:
                word_count += 1
                try:
                    swn_synset = swn.senti_synset(wn.synsets(word)[0].name())
                    if swn_synset.pos_score() > 0.0:
                        polarity_pos += 1
                    if swn_synset.neg_score() > 0.0:
                        polarity_neg += 1
                except:
                    pass
        polarity.append([polarity_pos,polarity_neg,word_count])
    return polarity


# Training

def train_naive_model(reviews_vectors,reviews_response):
    mnb = MultinomialNB()
    mnb.fit(reviews_vectors,reviews_response)
    return mnb


def train_regression_model(reviews_vectors,reviews_response):
    lrm = LogisticRegression(solver='liblinear', max_iter=1000)
    lrm.fit(reviews_vectors,reviews_response)
    return lrm
